@@ -58,8 +58,9 @@ from pw_console.console_prefs import ConsolePrefs
 from pw_console.help_window import HelpWindow
 import pw_console.key_bindings
 from pw_console.log_pane import LogPane
-from pw_console.python_logging import all_loggers
 from pw_console.pw_ptpython_repl import PwPtPythonRepl
+from pw_console.python_logging import all_loggers
+from pw_console.quit_dialog import QuitDialog
 from pw_console.repl_pane import ReplPane
 import pw_console.style
 import pw_console.widgets.checkbox
@@ -96,6 +97,25 @@ def _add_log_handler_to_pane(logger: Union[str, logging.Logger],
     pane.add_log_handler(logger, level_name=level_name)
 
 
+def get_default_colordepth(
+        color_depth: Optional[ColorDepth] = None) -> ColorDepth:
+    # Set prompt_toolkit color_depth to the highest possible.
+    if color_depth is None:
+        # Default to 24bit color
+        color_depth = ColorDepth.DEPTH_24_BIT
+
+        # If using Apple Terminal switch to 256 (8bit) color.
+        term_program = os.environ.get('TERM_PROGRAM', '')
+        if sys.platform == 'darwin' and 'Apple_Terminal' in term_program:
+            color_depth = ColorDepth.DEPTH_8_BIT
+
+    # Check for any PROMPT_TOOLKIT_COLOR_DEPTH environment variables
+    color_depth_override = os.environ.get('PROMPT_TOOLKIT_COLOR_DEPTH', '')
+    if color_depth_override:
+        color_depth = ColorDepth(color_depth_override)
+    return color_depth
+
+
 class ConsoleApp:
     """The main ConsoleApp class that glues everything together."""
 
@@ -111,19 +131,7 @@ class ConsoleApp:
         extra_completers=None,
     ):
         self.prefs = ConsolePrefs()
-        self.color_depth = color_depth
-        # Check for any PROMPT_TOOLKIT_COLOR_DEPTH environment variables
-        color_depth_override = os.environ.get('PROMPT_TOOLKIT_COLOR_DEPTH', '')
-
-        # Set prompt_toolkit color_depth to the highest possible.
-        if color_depth is None and not color_depth_override:
-            # Default to 24bit color
-            self.color_depth = ColorDepth.DEPTH_24_BIT
-
-            # If using Apple Terminal switch to 256 (8bit) color.
-            term_program = os.environ.get('TERM_PROGRAM', '')
-            if sys.platform == 'darwin' and 'Apple_Terminal' in term_program:
-                self.color_depth = ColorDepth.DEPTH_8_BIT
+        self.color_depth = get_default_colordepth(color_depth)
 
         # Create a default global and local symbol table. Values are the same
         # structure as what is returned by globals():
@@ -193,9 +201,6 @@ class ConsoleApp:
                                           title=(self.app_title + ' Help'))
         self.app_help_window.generate_help_text()
 
-        # Used for tracking which pane was in focus before showing help window.
-        self.last_focused_pane = None
-
         # Create a ptpython repl instance.
         self.pw_ptpython_repl = PwPtPythonRepl(
             get_globals=lambda: global_vars,
@@ -223,6 +228,8 @@ class ConsoleApp:
         # Top of screen menu items
         self.menu_items = self._create_menu_items()
 
+        self.quit_dialog = QuitDialog(self)
+
         # Key bindings registry.
         self.key_bindings = pw_console.key_bindings.create_key_bindings(self)
 
@@ -263,6 +270,11 @@ class ConsoleApp:
                     # Callable to get width
                     width=self.keybind_help_window.content_width,
                 ),
+                Float(
+                    content=self.quit_dialog,
+                    top=2,
+                    left=2,
+                ),
                 # Completion menu that can overlap other panes since it lives in
                 # the top level Float container.
                 Float(
@@ -312,6 +324,7 @@ class ConsoleApp:
                 # Pull key bindings from ptpython
                 load_python_bindings(self.pw_ptpython_repl),
                 load_sidebar_bindings(self.pw_ptpython_repl),
+                self.window_manager.key_bindings,
                 self.key_bindings,
             ]),
             style=DynamicStyle(lambda: merge_styles([
@@ -331,9 +344,20 @@ class ConsoleApp:
         return self.jinja_env.get_template(file_name)
 
     def run_pane_menu_option(self, function_to_run):
-        function_to_run()
+        # Run the function for a particular menu item.
+        return_value = function_to_run()
+        # It's return value dictates if the main menu should close or not.
+        # - True: The main menu stays open. This is the default prompt_toolkit
+        #   menu behavior.
+        # - False: The main menu closes.
+
+        # Update menu content. This will refresh checkboxes and add/remove
+        # items.
         self.update_menu_items()
-        self.focus_main_menu()
+        # Check if the main menu should stay open.
+        if not return_value:
+            # Keep the main menu open
+            self.focus_main_menu()
 
     def open_new_log_pane_for_logger(
             self,
@@ -434,6 +458,28 @@ class ConsoleApp:
                     MenuItem(
                         'Log Table View',
                         children=[
+                            MenuItem(
+                                '{check} Hide Date'.format(
+                                    check=pw_console.widgets.checkbox.
+                                    to_checkbox_text(
+                                        self.prefs.hide_date_from_log_time,
+                                        end='')),
+                                handler=functools.partial(
+                                    self.run_pane_menu_option,
+                                    functools.partial(
+                                        self.toggle_pref_option,
+                                        'hide_date_from_log_time')),
+                            ),
+                            MenuItem(
+                                '{check} Show Source File'.format(
+                                    check=pw_console.widgets.checkbox.
+                                    to_checkbox_text(
+                                        self.prefs.show_source_file, end='')),
+                                handler=functools.partial(
+                                    self.run_pane_menu_option,
+                                    functools.partial(self.toggle_pref_option,
+                                                      'show_source_file')),
+                            ),
                             MenuItem(
                                 '{check} Show Python File'.format(
                                     check=pw_console.widgets.checkbox.
@@ -471,10 +517,6 @@ class ConsoleApp:
             MenuItem(
                 '[Edit]',
                 children=[
-                    MenuItem('Copy visible lines from active window',
-                             handler=functools.partial(
-                                 self.window_manager.run_action_on_active_pane,
-                                 'copy_text')),
                     MenuItem('Paste to Python Input',
                              handler=self.repl_pane.
                              paste_system_clipboard_to_input_buffer),
@@ -486,6 +528,14 @@ class ConsoleApp:
             MenuItem(
                 '[View]',
                 children=[
+                    #         [Menu Item             ][Keybind  ]
+                    MenuItem('Focus Next Window/Tab   Ctrl-Alt-n',
+                             handler=self.window_manager.focus_next_pane),
+                    #         [Menu Item             ][Keybind  ]
+                    MenuItem('Focus Prev Window/Tab   Ctrl-Alt-p',
+                             handler=self.window_manager.focus_previous_pane),
+                    MenuItem('-'),
+
                     #         [Menu Item             ][Keybind  ]
                     MenuItem('Move Window Up         Ctrl-Alt-Up',
                              handler=functools.partial(
@@ -600,11 +650,16 @@ class ConsoleApp:
 
     def load_clean_config(self, config_file: Path) -> None:
         self.prefs.reset_config()
-        self.prefs.load_config(config_file)
+        self.prefs.load_config_file(config_file)
 
     def apply_window_config(self) -> None:
         self.window_manager.apply_config(self.prefs)
 
+    def refresh_layout(self) -> None:
+        self.window_manager.update_root_container_body()
+        self.update_menu_items()
+        self._update_help_window()
+
     def add_log_handler(
             self,
             window_title: str,
@@ -626,9 +681,7 @@ class ConsoleApp:
         for logger in logger_instances:
             _add_log_handler_to_pane(logger, existing_log_pane, log_level_name)
 
-        self.window_manager.update_root_container_body()
-        self.update_menu_items()
-        self._update_help_window()
+        self.refresh_layout()
         return existing_log_pane
 
     def _user_code_thread_entry(self):
@@ -664,6 +717,9 @@ class ConsoleApp:
         self.keybind_help_window.add_keybind_help_text('Global',
                                                        self.key_bindings)
 
+        self.keybind_help_window.add_keybind_help_text(
+            'Window Management', self.window_manager.key_bindings)
+
         # Add activated plugin key bindings to the help text.
         for pane in self.window_manager.active_panes():
             for key_bindings in pane.get_all_key_bindings():
@@ -688,12 +744,15 @@ class ConsoleApp:
         return self.application.layout.current_window
 
     def modal_window_is_open(self):
+        """Return true if any modal window or dialog is open."""
         if self.app_help_text:
             return (self.app_help_window.show_window
                     or self.keybind_help_window.show_window
-                    or self.user_guide_window.show_window)
+                    or self.user_guide_window.show_window
+                    or self.quit_dialog.show_dialog)
         return (self.keybind_help_window.show_window
-                or self.user_guide_window.show_window)
+                or self.user_guide_window.show_window
+                or self.quit_dialog.show_dialog)
 
     def exit_console(self):
         """Quit the console prompt_toolkit application UI."""
@@ -737,7 +796,8 @@ class ConsoleApp:
         # Fake module column names.
         module_names = ['APP', 'RADIO', 'BAT', 'USB', 'CPU']
         while True:
-            await asyncio.sleep(1)
+            if message_count > 32 or message_count < 2:
+                await asyncio.sleep(1)
             bar_size = 10
             position = message_count % bar_size
             bar_content = " " * (bar_size - position - 1) + "="
@@ -746,21 +806,23 @@ class ConsoleApp:
             new_log_line = 'Log message [{}] # {}'.format(
                 bar_content, message_count)
             if message_count % 10 == 0:
-                new_log_line += (" Lorem ipsum dolor sit amet, consectetur "
-                                 "adipiscing elit.") * 8
-            # TODO(tonymd): Add this in when testing log lines with included
-            # linebreaks.
-            # if message_count % 11 == 0:
-            #     new_log_line += inspect.cleandoc(""" [PYTHON] START
-            #         In []: import time;
-            #                 def t(s):
-            #                     time.sleep(s)
-            #                     return 't({}) seconds done'.format(s)""")
+                new_log_line += (
+                    ' Lorem ipsum \033[34m\033[1mdolor sit amet\033[0m'
+                    ', consectetur '
+                    'adipiscing elit.') * 8
+            if message_count % 11 == 0:
+                new_log_line += ' '
+                new_log_line += (
+                    '[PYTHON] START\n'
+                    'In []: import time;\n'
+                    '        def t(s):\n'
+                    '            time.sleep(s)\n'
+                    '            return "t({}) seconds done".format(s)\n\n')
 
             module_name = module_names[message_count % len(module_names)]
-            _FAKE_DEVICE_LOG.info(
-                new_log_line,
-                extra=dict(extra_metadata_fields=dict(module=module_name)))
+            _FAKE_DEVICE_LOG.info(new_log_line,
+                                  extra=dict(extra_metadata_fields=dict(
+                                      module=module_name, file='fake_app.cc')))
             message_count += 1
 
 
