@@ -150,7 +150,7 @@ class ConfigError(Exception):
     pass
 
 
-def result_func(glob_warnings):
+def result_func(glob_warnings=()):
     def result(status, *args):
         return _Result(status, *([str(x) for x in glob_warnings] + list(args)))
 
@@ -203,6 +203,7 @@ class EnvSetup(object):
         self._optional_submodules = []
         self._required_submodules = []
         self._virtualenv_system_packages = False
+        self._pw_packages = []
         self._root_variable = None
 
         self._config_file_name = getattr(config_file, 'name', 'config file')
@@ -270,6 +271,9 @@ class EnvSetup(object):
             os.path.join(self._project_root, x)
             for x in config.pop('cipd_package_files', ()))
 
+        for pkg in config.pop('pw_packages', ()):
+            self._pw_packages.append(pkg)
+
         virtualenv = config.pop('virtualenv', {})
 
         if virtualenv.get('gn_root'):
@@ -367,6 +371,7 @@ class EnvSetup(object):
         steps = [
             ('CIPD package manager', self.cipd),
             ('Python environment', self.virtualenv),
+            ('pw packages', self.pw_package),
             ('Host tools', self.host_tools),
         ]
 
@@ -409,7 +414,7 @@ Then use `set +x` to go back to normal.
             )
 
             start = time.time()
-            spin = spinner.Spinner()
+            spin = spinner.Spinner(self._quiet)
             with spin():
                 result = step(spin)
             stop = time.time()
@@ -424,6 +429,15 @@ Then use `set +x` to go back to normal.
             if not result.ok():
                 return -1
 
+            # Log the environment state at the end of each step for debugging.
+            log_dir = os.path.join(self._install_dir, 'logs')
+            if not os.path.isdir(log_dir):
+                os.makedirs(log_dir)
+            actions_json = os.path.join(
+                log_dir, 'post-{}.json'.format(name.replace(' ', '_')))
+            with open(actions_json, 'w') as outs:
+                self._env.json(outs)
+
         self._log('')
         self._env.echo('')
 
@@ -546,6 +560,37 @@ Then use `set +x` to go back to normal.
 
         return result(_Result.Status.DONE)
 
+    def pw_package(self, unused_spin):
+        """Install "default" pw packages."""
+
+        result = result_func()
+
+        if not self._pw_packages:
+            return result(_Result.Status.SKIPPED)
+
+        logdir = os.path.join(self._install_dir, 'packages')
+        if not os.path.isdir(logdir):
+            os.makedirs(logdir)
+
+        for pkg in self._pw_packages:
+            print('installing {}'.format(pkg))
+            cmd = ['pw', 'package', 'install', pkg]
+
+            log = os.path.join(logdir, '{}.log'.format(pkg))
+            try:
+                with open(log, 'w') as outs, self._env():
+                    print(*cmd, file=outs)
+                    subprocess.check_call(cmd,
+                                          cwd=self._project_root,
+                                          stdout=outs,
+                                          stderr=subprocess.STDOUT)
+            except subprocess.CalledProcessError:
+                with open(log, 'r') as ins:
+                    sys.stderr.write(ins.read())
+                    raise
+
+        return result(_Result.Status.DONE)
+
     def host_tools(self, unused_spin):
         # The host tools are grabbed from CIPD, at least initially. If the
         # user has a current host build, that build will be used instead.
