@@ -23,6 +23,12 @@ from typing import Callable, Collection, Optional, Sequence
 from pw_presubmit import git_repo, presubmit
 
 _LOG = logging.getLogger(__name__)
+DEFAULT_PATH = Path('out', 'presubmit')
+
+_OUTPUT_PATH_README = '''\
+This directory was created by pw_presubmit to run presubmit checks for the
+{repo} repository. This directory may be deleted safely.
+'''
 
 
 def add_path_arguments(parser) -> None:
@@ -35,12 +41,22 @@ def add_path_arguments(parser) -> None:
         help=('Paths or patterns to which to restrict the checks. These are '
               'interpreted as Git pathspecs. If --base is provided, only '
               'paths changed since that commit are checked.'))
-    parser.add_argument(
+
+    base = parser.add_mutually_exclusive_group()
+    base.add_argument(
         '-b',
         '--base',
         metavar='commit',
+        default=git_repo.TRACKING_BRANCH_ALIAS,
         help=('Git revision against which to diff for changed files. '
-              'If none is provided, the entire repository is used.'))
+              'Default is the tracking branch of the current branch.'))
+    base.add_argument(
+        '--full',
+        dest='base',
+        action='store_const',
+        const=None,
+        help='Run presubmit on all files, not just changed files.')
+
     parser.add_argument(
         '-e',
         '--exclude',
@@ -106,7 +122,7 @@ def add_arguments(parser: argparse.ArgumentParser,
     parser.add_argument(
         '--output-directory',
         type=Path,
-        help='Output directory (default: <repo root>/.presubmit)',
+        help=f'Output directory (default: {"<repo root>" / DEFAULT_PATH})',
     )
     parser.add_argument(
         '--package-root',
@@ -172,7 +188,11 @@ def run(
         repositories = [root]
 
     if output_directory is None:
-        output_directory = root / '.presubmit'
+        output_directory = root / DEFAULT_PATH
+
+    output_directory.mkdir(parents=True, exist_ok=True)
+    output_directory.joinpath('README.txt').write_text(
+        _OUTPUT_PATH_README.format(repo=root))
 
     if not package_root:
         package_root = output_directory / 'packages'
